import django_filters
from django import forms
from balder.enum import InputEnum
from graphene_django.forms.converter import convert_form_field


class EnumChoiceField(forms.CharField):

    def __init__(self, *args, choices=None, type=None, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.__choices = choices
        self.__type = type

    @property
    def overwritten_type(self):
        if self.__type: return self.__type
        return InputEnum.from_choices(self.__choices)



@convert_form_field.register(EnumChoiceField)
def convert_form_field_to_string_list(field):
    return field.overwritten_type(required=field.required)

class EnumFilter(django_filters.CharFilter):
    field_class = EnumChoiceField

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        assert self.field_name

    def filter(self, qs, value):
        """ Convert the filter value to a primary key before filtering """
        if value:
            return qs.filter(**{self.field_name: value})
        return qs



from abc import ABCMeta
from balder.registry import get_balder_registry
import logging

logger = logging.getLogger(__name__)

class BalderDefaultMutationMeta:
    abstract = False
    personal = None
    list = None
    filter = None
    operation = None
    queryset = None

class EmptyOverrideMeta:
    pass

class EmptyArguments:
    pass


class BalderMutationMeta(ABCMeta):

    def __new__(cls, name, bases, ns, **kwargs):

        return super(ABCMeta, cls).__new__(cls, name, bases, ns, **kwargs)

    def __init__(cls, name, bases, ns, **kwargs):
        super(ABCMeta, cls).__init__(name, bases, ns)
        meta_overrides = getattr(cls, "Meta", EmptyOverrideMeta)
        cls._meta = type("OverwrittenMeta", (meta_overrides, BalderDefaultMutationMeta), {"__doc__": f"Overwritten Meta for {cls.__name__}"})
        cls._arguments = getattr(cls, "Arguments", EmptyArguments)
        if not cls._meta.abstract:
            logger.info(f"Registering {name} as Mutation")
            get_balder_registry().registerMutation(cls)